rm(list=ls())

## ------------------------------------------------------------------------
#modification du dossier de travail
setwd("... votre dossier ...")

#importation des donnes
DToy <- read.table("toy_data.txt",sep="\t",header=T)
print(str(DToy))

## ------------------------------------------------------------------------
# predicteur par dfaut, moyenne de y
default_pred <- mean(DToy$y)

#calcul du MSE correspondant
default_mse <- mean((DToy$y-default_pred)^2)
print(default_mse)


## ------------------------------------------------------------------------
#rgression avec lm
regToy <- lm(y ~ ., data = DToy)

#rsultats dtaills
sregToy <- summary(regToy)
print(sregToy)

## ------------------------------------------------------------------------
#MSE pour la rgression avec lm()
lm_mse <- mean((DToy$y - regToy$fitted.values)^2)
print(lm_mse)

## ------------------------------------------------------------------------
#chargement de la librairie
library(gradDescent)

#appel de la fonction
mgd1 <- gradDescentR.learn(DToy,featureScaling = FALSE,learningMethod = "GD", seed = 1)

## ------------------------------------------------------------------------
#affichage du modle
print(mgd1$model)

## ------------------------------------------------------------------------
#fonction pour la mise en forme des coefficients
print.coefs <- function(coefs,D){
  coefs1 <- matrix(coefs,ncol=1,nrow=length(coefs))
  rownames(coefs1) <- c("intercept",colnames(D)[-ncol(D)])
  print(coefs1)
}
  
#affichage des coefficients de mgd1
print.coefs(mgd1$model,DToy)

## ------------------------------------------------------------------------
#calcul du mse
calc.mse <- function(D,coefs){
  #prdiction : application des coefficients estims sur les donnes
  yc <- apply(as.matrix(cbind(1,D[-ncol(D)])),1,function(x){sum(x*coefs)})
  #rsidu
  res <- D[ncol(D)]-yc
  #mse
  return(mean(res^2))
}

#MSE pour mgd1
print(calc.mse(DToy,mgd1$model))

## ------------------------------------------------------------------------
#passer le nombre d'itrations  maxIter = 1000
mgd2 <- gradDescentR.learn(DToy,featureScaling = FALSE,learningMethod="GD", seed = 1, control=list(maxIter=1000))

#calcul et affichage du mse
print(calc.mse(DToy,mgd2$model))

## ------------------------------------------------------------------------
#coefficients du second modle
print.coefs(mgd2$model,DToy)

## ------------------------------------------------------------------------
#fonction descente de gradient du package gradDescent
GD <- function(dataTrain, alpha=0.1, maxIter=10, seed=NULL){
	#convert data.frame dataSet in matrix
	dataTrain <- matrix(unlist(dataTrain), ncol=ncol(dataTrain), byrow=FALSE)
	#shuffle data train
	set.seed(seed)
	dataTrain <- dataTrain[sample(nrow(dataTrain)), ]
	set.seed(NULL)
	#initialize theta
	theta <- getTheta(ncol(dataTrain), seed=seed)
	#bind 1 column to dataTrain
	dataTrain <- cbind(1, dataTrain)
	#parse dataTrain into input and output
	inputData <- dataTrain[,1:ncol(dataTrain)-1]
	outputData <- dataTrain[,ncol(dataTrain)]
	#temporary variables
	temporaryTheta <- matrix(ncol=length(theta), nrow=1)
	#updateRule <- matrix(0, ncol=length(theta), nrow=1)
	#constant variables
	rowLength <- nrow(dataTrain)
	#loop the gradient descent
	for(iteration in 1:maxIter){
		error <- (inputData %*% t(theta)) - outputData
		for(column in 1:length(theta)){
			term <- error * inputData[,column]
			#calculate gradient
			gradient <- sum(term) / rowLength
			temporaryTheta[1,column] = theta[1,column] - (alpha*gradient)
		}
		#update all theta in the current iteration
		theta <- temporaryTheta
	}
	result <- theta
	return(result)
}

## ------------------------------------------------------------------------
#descente de gradient stochastique
msgd1 <- gradDescentR.learn(DToy,featureScaling = FALSE,learningMethod = "SGD", seed = 1)

#mse
print(calc.mse(DToy,msgd1$model))

## ------------------------------------------------------------------------
#stochastic gradient descent
SGD <- function(dataTrain, alpha=0.1, maxIter=10, seed=NULL){
	#convert data.frame dataSet in matrix
	dataTrain <- matrix(unlist(dataTrain), ncol=ncol(dataTrain), byrow=FALSE)
	#shuffle dataTrain
	set.seed(seed)
	dataTrain <- dataTrain[sample(nrow(dataTrain)), ]
	set.seed(NULL)
	#initialize theta
	theta <- getTheta(ncol(dataTrain), seed=seed)
	#bind 1 column to dataTrain
	dataTrain <- cbind(1, dataTrain)
	#parse dataTrain into input and output
	inputData <- dataTrain[,1:ncol(dataTrain)-1]
	outputData <- dataTrain[,ncol(dataTrain)]
	#temporary variables
	temporaryTheta <- matrix(ncol=length(theta), nrow=1)
	# updateRule <- matrix(0, ncol=length(theta), nrow=1)
	#constant variables
	rowLength <- nrow(dataTrain)
	set.seed(seed)
	stochasticList <- sample(1:rowLength, maxIter, replace=TRUE)
	set.seed(NULL)
	#loop the gradient descent
	for(iteration in 1:maxIter){
		error <- (inputData[stochasticList[iteration],] %*% t(theta)) - outputData[stochasticList[iteration]]
		for(column in 1:length(theta)){
			#calculate gradient
			gradient <- error * inputData[stochasticList[iteration], column]
			temporaryTheta[1,column] = theta[1,column] - (alpha*gradient)
		}
		#update all theta in the current iteration
		theta <- temporaryTheta
	}
	result <- theta
	return(result)
}

## ------------------------------------------------------------------------
#importation de la librairie keras
library(keras)

#structure de rseau de neurones
msgdk1 <- keras_model_sequential()

#une couche
msgdk1 %>%
  layer_dense(units=1,input_shape = c(10),activation="linear")

## ------------------------------------------------------------------------
#architecture du rseau
print(summary(msgdk1))

## ------------------------------------------------------------------------
#compilation
msgdk1 %>% compile(
  loss="mean_squared_error",
  optimizer=optimizer_sgd(),
  metrics="mean_squared_error"
)

## ------------------------------------------------------------------------
#construction du modle
msgdk1 %>% fit(
  x=as.matrix(DToy[,-ncol(DToy)]),
  y=DToy[,ncol(DToy)],
  epochs = 10,
  batch_size = 1
)

## ----message=FALSE, warning=FALSE----------------------------------------
#1000 itrations supplmentaires
msgdk1 %>% fit(
  x=as.matrix(DToy[,-ncol(DToy)]),
  y=DToy[,ncol(DToy)],
  epochs = 1000,
  batch_size = 10
)

## ------------------------------------------------------------------------
#paramtres estims du modle
print(get_weights(msgdk1))

