rm(list=ls())

#modification du dossier de travail
setwd("... votre dossier ...")

#importation des donnes
DProt <- read.table("protein01.txt",sep="\t",header=T)
print(dim(DProt))

#index pour la partition apprentissage-test
set.seed(1)
idTrain <- sample(1:nrow(DProt),51)

#rgression impossible, nombre de variables > nombre d'observations
regp <- lm(y ~ ., data = DProt[idTrain,])

#affichage des 50 premiers coefficients
print(regp$coefficients[1:50])

#liste des coefficients disponibles
print(regp$coefficients[!is.na(regp$coefficients)])

#keras
library(keras)

#structure vide de rseau
mgdkp <- keras_model_sequential()

#couche unique relaint l'entre  la sortie
mgdkp %>%
  layer_dense(units=1,input_shape = c(ncol(DProt)-1),activation="linear")

#vrification de l'architecture
print(summary(mgdkp))

#compilation
mgdkp %>% compile(
  loss="mean_squared_error",
  optimizer=optimizer_sgd(),
  metrics="mean_squared_error"
)

#apprentissage
mgdkp %>% fit(
  x=as.matrix(DProt[idTrain,-ncol(DProt)]),
  y=DProt[idTrain,ncol(DProt)],
  epochs = 100,
  batch_size = 32
)

#rcuparation des coefficients
coefs <- get_weights(mgdkp)[[1]][,1]

#associer les coefficients aux noms des variables
names(coefs) <- colnames(DProt)[-ncol(DProt)]

#affichage des coefs des 10 premires variables
print(coefs[1:10])

#nombre de N/A
print(length(which(is.na(coefs))))

#histogramme des coefficients
hist(coefs)

#40 coefficients les plsu levs en valeur absolue
print(sort(abs(coefs),decreasing=T)[1:40])


#prdiction sur l'chantillon test
ycp <- mgdkp %>% predict(as.matrix(DProt[-idTrain,-ncol(DProt)]))

#caractristiques des prdictions
print(summary(ycp))

#valeur de rfrence pour la prdiction
ref_mean <- mean(DProt$y[idTrain])
print(ref_mean)

#classement
cp <- ifelse(ycp > ref_mean,1,0)
print(table(cp))

#matrice de confusion
mc <- table(DProt$y[-idTrain],cp)
print(mc)

#taux d'erreur
err <- 1-sum(diag(mc))/sum(mc)
print(err)
